from distutils.core import setup
setup(name='cambank', author='Camille Huot', author_email='noemail@noemail.com',
  version = '2.3.5', url = 'https://github.com/camillehuot/cambank',
  description='CamBank is a personal finance Python module',
  packages = ['cambank'], package_dir= { 'cambank': 'cambank' }
 )
